@@ -21,9 +21,7 @@
 #' \itemize{
 #'  \item{1: \code{RRRStudio -> RRR_Studio}}
 #'  \item{2: \code{RRRStudio -> RRRS_tudio}}
-#'  \item{3: This \code{parsing_option} will suppress the conversion after non-alphanumeric
-#'  values. See examples.}
-#'  \item{4: digits directly behind/in front non-digits, will stay as is.}
+#'  \item{3: This \code{parsing_option} will suppress the conversion after non-alphanumeric values. See examples.}
 #'  \item{any other integer <= 0: no parsing"}
 #'  }
 #' 
@@ -41,7 +39,7 @@
 #' @param numerals A character specifying the alignment of numerals (\code{"middle"}, \code{left}, \code{right} or \code{asis}). I.e. \code{numerals = "left"} ensures that no output separator is in front of a digit.
 #' 
 #' @param sep_out (short for separator output) String that will be used as separator. The defaults are \code{"_"} 
-#' and \code{""}, regarding the specified \code{case}.
+#' and \code{""}, regarding the specified \code{case}. When \code{length(sep_out) > 1}, the last element of \code{sep_out} gets recycled and separators are incorporated per string according to their order.
 #' 
 #' @param unique_sep A string. If not \code{NULL}, then duplicated names will get 
 #' a suffix integer
@@ -81,6 +79,8 @@
 #' to_mixed_case(strings)
 #' to_swap_case(strings)
 #' to_sentence_case(strings)
+#' to_random_case(strings)
+#' to_title_case(strings)
 #' 
 #' 
 #' @rdname caseconverter
@@ -147,7 +147,8 @@ to_upper_camel_case <- function(string,
                                 abbreviations = NULL,
                                 sep_in = "[^[:alnum:]]",
                                 parsing_option = 1,
-                                transliterations = NULL,                           numerals = "middle",                                  
+                                transliterations = NULL,                           
+                                numerals = "middle",                                  
                                 sep_out = NULL,
                                 unique_sep = NULL,
                                 empty_fill = NULL,
@@ -174,7 +175,8 @@ to_screaming_snake_case <- function(string,
                                     abbreviations = NULL,
                                     sep_in = "[^[:alnum:]]",
                                     parsing_option = 1,
-                                    transliterations = NULL,                           numerals = "middle",                                  
+                                    transliterations = NULL,                      
+                                    numerals = "middle",                                  
                                     sep_out = NULL,
                                     unique_sep = NULL,
                                     empty_fill = NULL,
@@ -201,7 +203,8 @@ to_parsed_case <- function(string,
                            abbreviations = NULL,
                            sep_in = "[^[:alnum:]]",
                            parsing_option = 1,
-                           transliterations = NULL,                           numerals = "middle",                                  
+                           transliterations = NULL,                     
+                           numerals = "middle",                                  
                            sep_out = NULL,
                            unique_sep = NULL,
                            empty_fill = NULL,
@@ -341,7 +344,8 @@ to_sentence_case <- function(string,
                          sep_in = "[^[:alnum:]]",
                          parsing_option = 1,
                          transliterations = NULL,       
-                         numerals = "middle",                                             sep_out = NULL,
+                         numerals = "middle",                     
+                         sep_out = NULL,
                          unique_sep = NULL,
                          empty_fill = NULL,
                          prefix = "",
@@ -359,3 +363,59 @@ to_sentence_case <- function(string,
               parsing_option = parsing_option,
               abbreviations = abbreviations)
 }
+
+#' @rdname caseconverter
+#' @export
+
+to_random_case <- function(string,
+                         abbreviations = NULL,
+                         sep_in = "[^[:alnum:]]",
+                         parsing_option = 1,
+                         transliterations = NULL,             
+                         numerals = "middle",                   
+                         sep_out = NULL,
+                         unique_sep = NULL,
+                         empty_fill = NULL,
+                         prefix = "",
+                         postfix = ""){
+  to_any_case(string = string,
+              case = "random",
+              sep_in = sep_in,
+              transliterations = transliterations,              
+              numerals = numerals,
+              sep_out = sep_out,
+              prefix = prefix,
+              postfix = postfix,
+              unique_sep = unique_sep,
+              empty_fill = empty_fill,
+              parsing_option = parsing_option,
+              abbreviations = abbreviations)
+}
+
+#' @rdname caseconverter
+#' @export
+
+to_title_case <- function(string,
+                         abbreviations = NULL,
+                         sep_in = "[^[:alnum:]]",
+                         parsing_option = 1,
+                         transliterations = NULL,             
+                         numerals = "middle",                   
+                         sep_out = NULL,
+                         unique_sep = NULL,
+                         empty_fill = NULL,
+                         prefix = "",
+                         postfix = ""){
+  to_any_case(string = string,
+              case = "title",
+              sep_in = sep_in,
+              transliterations = transliterations,              
+              numerals = numerals,
+              sep_out = sep_out,
+              prefix = prefix,
+              postfix = postfix,
+              unique_sep = unique_sep,
+              empty_fill = empty_fill,
+              parsing_option = parsing_option,
+              abbreviations = abbreviations)
+}
