@@ -12,6 +12,8 @@
 #'  \item{ALL_CAPS: \code{"all_caps"} or \code{"screaming_snake"}}
 #'  \item{lowerUPPER: \code{"lower_upper"}}
 #'  \item{UPPERlower: \code{"upper_lower"}}
+#'  \item{Sentence case: \code{"sentence"}}
+#'  \item{Title Case: \code{"title"} - This one is basically the same as "parsed" case below, but in addition it is wrapped into \code{tools::toTitleCase}.}
 #'  }
 #'
 #'  There are five "special" cases available:
@@ -25,9 +27,10 @@
 #'  or behind an underscore is turned into lowercase. If a substring is set as an abbreviation, it will stay in upper case.}
 #'  \item{\code{"swap"}: Upper case letters will be turned into lower case and vice versa. Also \code{case = "flip"} will work.
 #'  Doesn't work with any of the other arguments except \code{unique_sep}, \code{empty_fill}, \code{prefix} and \code{postfix}.}
+#'  \item{\code{"random"}: Each letter will be randomly turned into lower or upper case. Doesn't work with any of the other arguments except \code{unique_sep}, \code{empty_fill}, \code{prefix} and \code{postfix}.}
 #'  \item{\code{"none"}: Neither parsing nor case conversion occur. This case might be helpful, when
 #'  one wants to call the function for the quick usage of the other parameters.
-#'  To suppress replacement of spaces to underscores set \code{sep_out = NULL}.
+#'  To suppress replacement of spaces to underscores set \code{sep_in = NULL}.
 #'  Works with \code{sep_in}, \code{transliterations}, \code{sep_out}, \code{prefix},
 #'   \code{postfix},
 #'   \code{empty_fill} and \code{unique_sep}.}
@@ -53,9 +56,7 @@
 #' \itemize{
 #'  \item{1: \code{RRRStudio -> RRR_Studio}}
 #'  \item{2: \code{RRRStudio -> RRRS_tudio}}
-#'  \item{3: This \code{parsing_option} will suppress the conversion after non-alphanumeric
-#'  values. See examples.}
-#'  \item{4: digits directly behind/in front non-digits, will stay as is.}
+#'  \item{3: This \code{parsing_option} will suppress the conversion after non-alphanumeric values. See examples.}
 #'  \item{any other integer <= 0: no parsing"}
 #'  }
 #'
@@ -63,15 +64,17 @@
 #' need to be elements of \code{stringi::stri_trans_list()} (like "Latin-ASCII", which is often useful) or names of lookup tables (currently
 #' only "german" is supported). In the order of the entries the letters of the input
 #'  string will be transliterated via \code{stringi::stri_trans_general()} or replaced via the 
-#'  matches of the lookup table.
-#'  
+#'  matches of the lookup table. When named character elements are supplied as part of `transliterations`, anything that matches the names is replaced by the corresponding value.
+
 #' You should use this feature with care in case of \code{case = "parsed"}, \code{case = "internal_parsing"} and 
 #' \code{case = "none"}, since for upper case letters, which have transliterations/replacements
 #'  of length 2, the second letter will be transliterated to lowercase, for example Oe, Ae, Ss, which
-#'  might not always be what is intended.
+#'  might not always be what is intended. In this case you can make usage of the option to supply named elements and specify the transliterations yourself.
+#' 
+#' @param numerals A character specifying the alignment of numerals (\code{"middle"}, \code{left}, \code{right}, \code{asis} or \code{tight}). I.e. \code{numerals = "left"} ensures that no output separator is in front of a digit.
 #' 
 #' @param sep_out (short for separator output) String that will be used as separator. The defaults are \code{"_"} 
-#' and \code{""}, regarding the specified \code{case}.
+#' and \code{""}, regarding the specified \code{case}. When \code{length(sep_out) > 1}, the last element of \code{sep_out} gets recycled and separators are incorporated per string according to their order.
 #' 
 #' @param unique_sep A string. If not \code{NULL}, then duplicated names will get 
 #' a suffix integer
@@ -102,6 +105,7 @@
 #' to_any_case(strings, case = "all_caps") # same as "screaming_snake"
 #' to_any_case(strings, case = "lower_upper")
 #' to_any_case(strings, case = "upper_lower")
+#' to_any_case(strings, case = "sentence")
 #' to_any_case(strings, case = "parsed")
 #' to_any_case(strings, case = "mixed")
 #' to_any_case(strings, case = "internal_parsing")
@@ -116,8 +120,8 @@
 #' # This option (3) suppresses this behaviour
 #' to_any_case("blaBla.bla", case = "big_camel", parsing_option = 3)
 #' # Numbers are always separated from characters via an underscore. 
-#' # To suppress this behaviour choose parsing_option 4
-#' snakecase::to_any_case("d3 library", case = "upper_camel", parsing_option = 4, sep_out = " ")
+#' # To suppress this behaviour choose \code{numerals = "tight"}
+#' to_any_case("d3 library", case = "upper_camel", sep_out = " ", numerals = "tight")
 #' # there might be reasons to suppress the parsing, while choosing neither one or two
 #' to_any_case("HAMBURGcity", case = "parsed", parsing_option = 0)
 #' 
@@ -157,96 +161,112 @@ to_any_case <- function(string,
                         case = c("snake", "small_camel", "big_camel", "screaming_snake", 
                                  "parsed", "mixed", "lower_upper", "upper_lower", "swap",
                                  "all_caps", "lower_camel", "upper_camel", "internal_parsing", 
-                                 "none", "flip"),
+                                 "none", "flip", "sentence", "random", "title"),
                         abbreviations = NULL,
                         sep_in = "[^[:alnum:]]",
                         parsing_option = 1,
                         transliterations = NULL,
+                        numerals = c("middle", "left", "right", "asis", "tight"),
                         sep_out = NULL,
                         unique_sep = NULL,
                         empty_fill = NULL,
                         prefix = "",
                         postfix = ""){
-  ### Deprecations:
-  # if (!identical(preprocess, NULL)) {
-  #   warning("argument preprocess is renamed to sep_in and will be removed in later versions",
-  #           call. = FALSE)
-  #   if (identical(sep_in, NULL)) {
-  #     sep_in = preprocess
-  #   }
-  # }
-  ### Argument matching and checking
+### ____________________________________________________________________________
+### Argument matching
   case <- match.arg(case)
-### check input length (necessary for NULL and atomic(0))
-  if(identical(stringr::str_length(string), integer())){return(character())}
+  numerals <- match.arg(numerals)
 ### ____________________________________________________________________________
-### save names-attribute
-  string_names <- names(string)
+### Set encoding to utf8
+  string <- enc2utf8(string)
 ### ____________________________________________________________________________
-### helper for "lower_upper", "upper_lower"
-  # this helper returns a logical vector with TRUE for the first and every
-  # second string of those which contain an alphabetic character
-  if(case == "upper_lower" | case == "lower_upper") {
-    relevant <- function(string){
-      relevant <- stringr::str_detect(string, "[:alpha:]")
-      relevant[relevant] <- rep_len(c(TRUE, FALSE), sum(relevant))
-      relevant
-    }
-  }
-### Aliases
+### Argument checking (check input length -> necessary for NULL and atomic(0))
+  if (identical(stringr::str_length(string), integer())) {return(character())}
+### ____________________________________________________________________________
+### Save attributes
+  string_attributes <- attributes(string)
+### ____________________________________________________________________________
+### Handle aliases
   case[case == "all_caps"] <- "screaming_snake"
   case[case == "lower_camel"] <- "small_camel"
   case[case == "upper_camel"] <- "big_camel"
   case[case == "flip"] <- "swap"
+### ____________________________________________________________________________
+### Prepare title case
+  title <- if (case == "title") TRUE else FALSE
+  case[case == "title"] <- "parsed"
+### ____________________________________________________________________________
 ### Handle swap case
-  
   if (case == "swap") {
     string <- gsub(pattern = "([[:upper:]])|([[:lower:]])",
                    perl = TRUE,
                    replacement = "\\L\\1\\U\\2",
-                   string)
+                   string)}
+### ____________________________________________________________________________
+### Handle random case
+  if (case == "random") {
+    random_case <- function(string) {
+      upper_or_lower <- function(string) {
+        if(sample(c(TRUE, FALSE), 1)) {return(toupper(string))}
+        tolower(string)
+      }
+      
+      unlist(
+        lapply(
+          strsplit(string, split = character(0)),
+          function(x) paste0(unlist(lapply(x, upper_or_lower)), collapse = "")
+        )
+      )
+    }
+    
+    string <- random_case(string)
   }
-  
-### abbreviation handling
-  # mark abbreviation with an underscore behind (in front of the parsing)
-  # useful if parsinoption 1 is needed, but some abbreviations need parsing_option 2
-  if (case != "swap"){
+### ____________________________________________________________________________
+### Match abbreviations
+  # mark abbreviation by placing an underscore behind them (in front of the parsing)
+  # useful if parsingoption 1 is needed, but some abbreviations need parsing_option 2
+  if (!case %in% c("swap", "random")) {
     string <- abbreviation_internal(string, abbreviations)
   }
 ### ____________________________________________________________________________
-### preprocess (regex into "_") and parsing (surrounding by "_")
-if (case != "swap") {
+### Preprocessing:
+## Turn mateches of `sep_in` into "_" and
+## surround matechs of the parsing by "_" (parsed_case)
+  if (!case %in% c("swap", "random")) {
     string <- preprocess_internal(string, sep_in)
-  
-  if (!case %in% c("none")){
-    string <- to_parsed_case_internal(string,
-                                      parsing_option = parsing_option)
-  } else {
-    string <- vapply(string, stringr::str_replace_all, "","_+", "_", 
-                     USE.NAMES = FALSE) 
-    string <- vapply(string, stringr::str_replace_all, "","^_|_$", "",
-                     USE.NAMES = FALSE)
-  }
+    
+    if (case != "none") {
+      string <- to_parsed_case_internal(string,
+                                        parsing_option = parsing_option,
+                                        numerals = numerals)
+    } else {
+      string <- vapply(string, stringr::str_replace_all, "","_+", "_", 
+                       USE.NAMES = FALSE) 
+      string <- vapply(string, stringr::str_replace_all, "","^_|_$", "",
+                       USE.NAMES = FALSE)
+    }
 ### ____________________________________________________________________________
 ### "mixed", "snake", "small_camel", "big_camel", "screaming_case", "parsed"
-  if(case %in% c("mixed", "snake", "small_camel",
+  if (case %in% c("mixed", "snake", "small_camel",
                  "big_camel", "screaming_snake", "parsed",
-                 "lower_upper", "upper_lower")){
+                 "lower_upper", "upper_lower", "sentence")) {
 ### split-----------------------------------------------------------------------
-    if(case %in% c("mixed", "snake", "screaming_snake", "parsed", "lower_upper", "upper_lower")){
+    if (case %in% c("mixed", "snake", "screaming_snake", "parsed", "lower_upper", "upper_lower", "sentence")) {
       string <- stringr::str_split(string, "_")
     }
     #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
-    if(case %in% c("small_camel", "big_camel")){
+    if (case %in% c("small_camel", "big_camel")) {
       string <- stringr::str_split(string, pattern = "(?<!\\d)_|_(?!\\d)")
     }
 ### replacement of special characters_------------------------------------------
-    if(!is.null(transliterations)){
-      string <- lapply(string, function (x)
+    if (!is.null(transliterations)) {
+      string <- lapply(string, function(x)
         replace_special_characters_internal(x, transliterations, case))
+      string <- lapply(string, function(x) 
+        stringr::str_replace_all(x, "_+", "_"))
     }
 ### caseconversion--------------------------------------------------------------
-    if(case == "mixed"){
+    if (case == "mixed") {
       string <- lapply(string,
                        function(x) ifelse(!x %in% abbreviations, 
                            stringr::str_c(stringr::str_sub(x, 1, 1),
@@ -255,13 +275,13 @@ if (case != "swap") {
         )
       }
     #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
-    if(case == "snake"){
+    if (case %in% c("snake", "sentence")) {
       string <- lapply(string, stringr::str_to_lower)
     }
     #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
-    if(case == "big_camel"){
+    if (case == "big_camel") {
       string <- lapply(string, stringr::str_to_lower)
-      if(!is.null(abbreviations)){
+      if (!is.null(abbreviations)) {
       string <- lapply(string, 
                        function(x) ifelse(x %in% stringr::str_to_lower(abbreviations), 
                                           stringr::str_to_upper(x),
@@ -273,7 +293,7 @@ if (case != "swap") {
                                                   stringr::str_sub(x, 2)))
     }
     #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
-    if(case == "small_camel"){
+    if (case == "small_camel") {
       string <- lapply(string, stringr::str_to_lower)
       string <- lapply(string, 
                        function(x) stringr::str_c(stringr::str_to_upper(stringr::str_sub(x, 1, 1)),
@@ -286,7 +306,7 @@ if (case != "swap") {
                                         stringr::str_sub(x, 2)), "",
                USE.NAMES = FALSE)
       string <- stringr::str_split(string, " ")
-      if(!is.null(abbreviations)){ # handle abbreviations
+      if (!is.null(abbreviations)) { # handle abbreviations
       string <- lapply(string,
                        function(x) c(stringr::str_to_lower(x[1]), 
                                      ifelse(stringr::str_to_lower(x[-1]) %in% stringr::str_to_lower(abbreviations),
@@ -294,11 +314,11 @@ if (case != "swap") {
       }
     }
     #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
-    if(case == "screaming_snake"){
+    if (case == "screaming_snake") {
       string <- lapply(string, stringr::str_to_upper)
     }
     #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
-    if (case == "lower_upper"){
+    if (case == "lower_upper") {
       string[!is.na(string)] <- mapply(function(x, y) { 
         # odds to lower
         x[y] <- stringr::str_to_lower(x[y]);
@@ -324,8 +344,23 @@ if (case != "swap") {
         USE.NAMES = TRUE)
     }
 ### collapsing------------------------------------------------------------------
-    if(case %in% c("none", "mixed", "snake", "screaming_snake", "parsed",
-                   "small_camel", "big_camel", "lower_upper", "upper_lower")) {
+    if (case %in% c("none", "mixed", "snake", "screaming_snake", 
+                   "small_camel", "big_camel", "lower_upper", "upper_lower") | 
+        (case == "parsed" & !title)) {
+      string <- vapply(string, 
+                       function(x) stringr::str_c(x, collapse = "_"), "",
+                       USE.NAMES = FALSE)
+    }
+    
+    if (case == "parsed" & title){
+      string <- vapply(string, 
+                       function(x) stringr::str_c(x, collapse = " "), "",
+                       USE.NAMES = FALSE)
+      string <- tools::toTitleCase(string)
+      string <- stringr::str_replace_all(string, " ", "_")
+    }
+    
+    if (case == "sentence") {
       string <- vapply(string, 
                        function(x) stringr::str_c(x, collapse = "_"), "",
                        USE.NAMES = FALSE)
@@ -333,50 +368,90 @@ if (case != "swap") {
     #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
     # Protect (only internal, not via an argument).
     # Replace all "_" by "" which are around a not alphanumeric character
-    if (parsing_option == 4){
+    if (parsing_option == 4 | numerals == "asis") {
       string <- stringr::str_replace_all(string, " ", "")
     }
+    if (numerals == "right" | numerals == "tight") {
+      # underscore with a digit before and no digit after
+      string <- stringr::str_replace_all(string, "(?<=\\d)_(?!\\d)", "")
+    }
     
-    if (case != "internal_parsing"){
+    if (numerals == "left" | numerals == "tight") {
+      # underscore with no digit before and a digit after
+      string <- stringr::str_replace_all(string, "(?<!\\d)_(?=\\d)", "")
+    }
+    
+    # omit (old) protection mode
+    if (case != "internal_parsing") {
       string <- stringr::str_replace_all(string, "_(?![:alnum:])|(?<![:alnum:])_", "")
     }
 ### ----------------------------------------------------------------------------
 }
 ### postprocessing (ouput separator)s--------------------------------------------
-    if(!is.null(sep_out) & !identical(string, character(0))){
-      string <- mapply(function(x, y) 
-        stringr::str_replace_all(x, "_", y),
-                                 string,
-                                 sep_out,
-        USE.NAMES = FALSE)
+    # if (!is.null(sep_out) & !identical(string, character(0))) {
+    #   string <- mapply(function(x, y) 
+    #     stringr::str_replace_all(x, "_", y),
+    #                              string,
+    #                              sep_out,
+    #     USE.NAMES = FALSE)
+    #   }
+    if (!is.null(sep_out) & !identical(string, character(0))) {
+      paste_along <- function(x, along = "_") {
+        if (length(x) <= 1L) return(x)
+        if (length(along) == 1L) return(paste0(x, collapse = along))
+        
+        along <- c(along, rep_len(along[length(along)], max(length(x) - length(along), 0L)))
+        paste0(paste0(x[seq_len(length(x) - 1)], along[seq_len(length(x) - 1)] ,
+                      collapse = ""), x[length(x)])
       }
+      
+      string <- stringr::str_split(string, pattern = "_")
+      string <- vapply(string, 
+                       function(x) paste_along(x, along = sep_out), "",
+                       USE.NAMES = FALSE)
+    }
     
-    if(is.null(sep_out) & case %in% c("small_camel", "big_camel", 
-                                            "lower_upper", "upper_lower")){
+    if (is.null(sep_out) & case %in% c("small_camel", "big_camel", 
+                                            "lower_upper", "upper_lower")) {
       string <- stringr::str_replace_all(string, "(?<!\\d)_|_(?!\\d)", "")
     }
+    
+    if (is.null(sep_out) & (case == "sentence" | (case == "parsed" & title))) {
+      string <- stringr::str_replace_all(string, "_", " ")
+    }
+
+    if (case == "sentence") {
+      string <- vapply(string, 
+                       function(x) stringr::str_c(stringr::str_to_upper(stringr::str_sub(x, 1, 1)),
+                                                  stringr::str_sub(x, 2)), "",
+                       USE.NAMES = FALSE)
+    }
 ### ____________________________________________________________________________
 ### "none"
-  if(case == "none" & !is.null(transliterations)){
-    string <- replace_special_characters_internal(string, transliterations)
+  if (case == "none" & !is.null(transliterations)) {
+    string <- replace_special_characters_internal(string, transliterations, case = case)
   }
 ### ____________________________________________________________________________    
 } # close swap
 ### ____________________________________________________________________________
 ### fill empty strings
-  if(!is.null(empty_fill) & any(string == "")){
+  if (!is.null(empty_fill) & any(string == "")) {
     string[string == ""] <- empty_fill
   }
 ### ____________________________________________________________________________
 ### make unique
-  if(!is.null(unique_sep))
+  if (!is.null(unique_sep)) {
     string <- make.unique(string, sep = unique_sep)
+  }
 ### ____________________________________________________________________________
 ### pre and postfix
   string <- stringr::str_c(prefix, string, postfix)
 ### ____________________________________________________________________________
-### set back names-attribute
-  names(string) <- string_names
+### set back attributes
+  attributes(string) <- string_attributes
+### ____________________________________________________________________________
+### guarantee that output is really UTF-8
+  string <- enc2utf8(string)
 ### ____________________________________________________________________________
 ### return
   string
