@@ -60,6 +60,145 @@ test_that("examples", {
   }
 )
 
+test_that("numerals_tight",
+          {
+            expect_equal(to_any_case("bla_la_123_123_1_bla", numerals = "tight"),
+                         "bla_la123_123_1bla")
+          })
+
+test_that("attributes", {
+  expect_equal(
+    {strings <- c("this Is a Strange_string", "AND THIS ANOTHER_One");
+    names(strings) <- c("String A", "String B");
+    attr(strings, "test.attr") <- "test";
+    strings},
+    structure(c(`String A` = "this Is a Strange_string", `String B` = "AND THIS ANOTHER_One"),
+              test.attr = "test"))
+    }
+  )
+
+test_that("numerals", {
+  expect_equal(to_any_case("123bla123bla_434bla"),
+               "123_bla_123_bla_434_bla")
+  
+  expect_equal(to_any_case("123bla123bla_434bla", 
+                           numerals = "asis"),
+               "123bla123bla_434bla")
+  
+  expect_equal(to_any_case("123bla123bla_434bla",
+                           numerals = "left"),
+               "123_bla123_bla434_bla")
+
+  expect_equal(to_any_case("123bla123bla_434bla", 
+                           numerals = "right"),
+               "123bla_123bla_434bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla", numerals = "middle"),
+               "123_bla_123_123_bla_434_bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla", numerals = "asis"),
+               "123bla123_123bla_434bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla", numerals = "left"),
+               "123_bla123_123_bla434_bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla",
+                           numerals = "right"),
+               "123bla_123_123bla_434bla")
+
+  expect_equal(to_any_case("123bla123_123bla_434bla", numerals = "right"), 
+               "123bla_123_123bla_434bla")
+
+  expect_equal(to_any_case("123bla123_123bla_434bla", numerals = "right"),
+               "123bla_123_123bla_434bla")
+  
+  expect_equal(to_upper_camel_case("123bla123_123bla_434bla"),
+               "123Bla123_123Bla434Bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla", case = "upper_camel", numerals = "middle"),
+    "123Bla123_123Bla434Bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla", case = "upper_camel", numerals = "asis"), "123Bla123_123Bla434Bla")
+
+  expect_equal(to_any_case("123bla123_123bla_434bla", case = "upper_camel", numerals = "left"), 
+               "123Bla123_123Bla434Bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla", case = "upper_camel", numerals = "right"),
+               "123Bla123_123Bla434Bla")
+
+  expect_equal(to_any_case("123bla123_123bla_434bla", case = "parsed", numerals = "middle"),
+               "123_bla_123_123_bla_434_bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla", case = "parsed", numerals = "asis"),
+               "123bla123_123bla_434bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla", case = "parsed", numerals = "left"),
+               "123_bla123_123_bla434_bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla",
+                           case = "parsed",
+                           numerals = "right"),
+    "123bla_123_123bla_434bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla", 
+                           case = "none",
+                           numerals = "middle"),
+               "123bla123_123bla_434bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla", 
+                           case = "none", 
+                           numerals = "asis"),
+               "123bla123_123bla_434bla")
+
+  expect_equal(to_any_case("123bla123_123bla_434bla",
+                           case = "none",
+                           numerals = "left"),
+               "123bla123_123bla_434bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla", 
+                           case = "none",
+                           numerals = "right"),
+               "123bla123_123bla_434bla")
+
+  expect_equal(to_any_case("123bla123_123bla_434bla", 
+                           numerals = "middle"),
+               "123_bla_123_123_bla_434_bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla", numerals = "asis"),
+               "123bla123_123bla_434bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla",
+                           numerals = "left"),
+               "123_bla123_123_bla434_bla")
+  
+  expect_equal(to_any_case("123bla123_123bla_434bla",
+                           numerals = "right"),
+               "123bla_123_123bla_434bla")
+  
+  expect_equal(to_any_case("species42value 23month",
+                           numerals = "asis"),
+               "species42value_23month")
+  
+  expect_equal(to_any_case(c("HHcity", "IDtable1", "KEYtable2", "newUSelections"),
+                           parsing_option = 2,
+                           numerals = "middle"),
+               c("hh_city", "id_table_1", "key_table_2", "new_us_elections"))
+  
+  expect_equal(to_any_case(c("HHcity", "IDtable1", "KEYtable2", "newUSelections"),
+                           parsing_option = 2, 
+                           numerals = "asis"),
+               c("hh_city","id_table1", "key_table2", "new_us_elections"))
+  
+  expect_equal(to_any_case(c("HHcity", "IDtable1", "KEYtable2", "newUSelections"), parsing_option = 2, numerals = "left"),
+               c("hh_city", "id_table1", "key_table2","new_us_elections"))
+
+  expect_equal(to_any_case(c("HHcity", "IDtable1", "KEYtable2", "newUSelections"),
+                           parsing_option = 2,
+                           numerals = "right"),
+               c("hh_city", "id_table_1", "key_table_2",
+                 "new_us_elections"))
+})
+
 test_that("preserve-names-attribute", {
   
   labs <- c(a = "abcDEF", b = "bbccEE", c = "TeESt it")
@@ -110,7 +249,27 @@ test_that("preserve-names-attribute", {
     )
 })
 
+test_that("uniqe_sep", {
+  expect_equal(
+    to_any_case(c("bla", "bla"), unique_sep = "_"),
+    c("bla",   "bla_1")
+  )
+  
+  expect_equal(
+    to_any_case(c("bla", "bla"), unique_sep = NULL),
+    c("bla", "bla")
+  )
+})
+
+test_that("sentence_case", {
+  expect_equal(
+  to_sentence_case("this_is_a_sentence", sep_out = " ", postfix = "."),
+  "This is a sentence.")
+})
+
 test_that("janitor-pkg-tests",{
+  skip_if_not( l10n_info()$`UTF-8`)
+  
   clean_names3 <- function(old_names, case = "snake"){
     new_names <- gsub("'", "", old_names) # remove quotation marks
     new_names <- gsub("\"", "", new_names) # remove quotation marks
@@ -236,16 +395,31 @@ test_that("random examples",
 #           expect_warning(to_any_case("bla", protect = "_"),
 #                          "argument protect is deprecated; If you really need this argument, pls submit an issue on https://github.com/Tazinho/snakecase")
 #           )
+
+test_that("transliterations", {
+  
+  skip_if_not( l10n_info()$`UTF-8`)
+  
+  expect_equal(to_any_case("Älterer Herr", transliterations = c("german", "Herr" = "Mann")), "aelterer_mann")
+  
+  expect_equal(
+    to_any_case("Älterer Herr", transliterations = c( "Herr" = "Mann", "german")),
+    "aelterer_mann")
+})
+
 test_that("transliterations_error", 
           expect_error(to_any_case("bla", transliterations = "bla"),
-                       "Input to `transliterations` must be `NULL` or a string containing elements
-               from the internal lookup dictionaries or from `stringi::stri_trans_list()`.",
+                       "Input to `transliterations` must be `NULL`, a string containing elements from the internal lookup dictionaries or from `stringi::stri_trans_list()` or a named vector.",
                        fixed = TRUE))
 
 test_that("empty_fill",
           expect_equal(to_any_case("", empty_fill = "bla"),
           "bla"))
 
+test_that("sentence",
+          expect_equal(to_any_case("bla bla_bal", case = "sentence"),
+                       "Bla bla bal"))
+  
 test_that("flip and swap", {
           expect_equal(to_any_case("rSTUDIO", case = "flip"), "Rstudio")
           
@@ -253,6 +427,9 @@ test_that("flip and swap", {
 })
 
 test_that("complex strings", {
+  
+  skip_if_not( l10n_info()$`UTF-8`)
+  
   strings2 <- c("this - Is_-: a Strange_string", "\u00C4ND THIS ANOTHER_One")
   
   expect_equal(to_any_case(strings2, case = "snake", sep_in = "-|\\:"),
@@ -346,6 +523,41 @@ test_that("complex strings", {
   
   expect_equal(to_any_case("ac\u00C4o", transliterations = c("german", "Latin-ASCII")),
                "ac_aeo")
+  #
+  expect_equal(to_any_case("\u00C6", transliterations = c("danish", "Latin-ASCII")),
+               "ae")
+  expect_equal(to_any_case("\u00E6", transliterations = c("danish", "Latin-ASCII")),
+               "ae")
+  expect_equal(to_any_case("\u00D8", transliterations = c("danish", "Latin-ASCII")),
+               "oe")
+  expect_equal(to_any_case("\u00F8", transliterations = c("danish", "Latin-ASCII")),
+               "oe")
+  expect_equal(to_any_case("\u00C5", transliterations = c("danish", "Latin-ASCII")),
+               "aa")
+  expect_equal(to_any_case("\u00E5", transliterations = c("danish", "Latin-ASCII")),
+               "aa")
+  
+  expect_equal(to_any_case("\u00C6", transliterations = c("finnish", "Latin-ASCII")),
+               "a")
+  expect_equal(to_any_case("\u00E6", transliterations = c("finnish", "Latin-ASCII")),
+               "a")
+  expect_equal(to_any_case("\u00D8", transliterations = c("finnish", "Latin-ASCII")),
+               "o")
+  expect_equal(to_any_case("\u00F8", transliterations = c("finnish", "Latin-ASCII")),
+               "o")
+  
+  expect_equal(to_any_case("\u00C6", transliterations = c("swedish")),
+               "a")
+  expect_equal(to_any_case("\u00E6", transliterations = c("swedish")),
+               "a")
+  expect_equal(to_any_case("\u00D8", transliterations = c("swedish")),
+               "o")
+  expect_equal(to_any_case("\u00F8", transliterations = c("swedish")),
+               "o")
+  expect_equal(to_any_case("\u00C5", transliterations = c("swedish")),
+               "a")
+  expect_equal(to_any_case("\u00E5", transliterations = c("swedish")),
+               "a")
   
   expect_equal(to_any_case("\u00E6", transliterations = "Latin-ASCII"),
                 "ae")
@@ -385,14 +597,11 @@ test_that("complex strings", {
   expect_equal(to_any_case("some11 21 31numbers_11 21 32_With11 11_diffs11 22 d", case = "lower_upper"),
                "some11_21_31NUMBERS11_21_32with11_11DIFFS11_22d")
   
-  expect_equal(to_any_case("email22Version_33 test", parsing_option = 4),
-               "email22version_33_test")
-  
 })
 
 
 test_that("stackoverflow answers", {
-  expect_equal(to_any_case(c("ThisText", "NextText"), case = "snake", sep_out = "\\."),
+  expect_equal(to_any_case(c("ThisText", "NextText"), case = "snake", sep_out = "."),
                c("this.text", "next.text"))
   
   expect_equal(to_any_case(c("BobDylanUSA",
@@ -683,4 +892,53 @@ test_that("expand.grid", {
                  "start.ICU_DAYS.end", "start.SEX_CODE.end", "start.MAX_OF_MLD.end", 
                  "start.AGE.GROUP.end"))
   }
-)
\ No newline at end of file
+)
+
+
+
+test_that("sep_out", {
+  paste_along <- function(x, along = "_") {
+    if (length(x) <= 1L) return(x)
+    if (length(along) == 1L) return(paste0(x, collapse = along))
+    
+    along <- c(along, rep_len(along[length(along)], max(length(x) - length(along), 0L)))
+    paste0(paste0(x[seq_len(length(x) - 1)], along[seq_len(length(x) - 1)],
+                  collapse = ""), x[length(x)])
+  }
+
+  expect_equal(to_any_case("a", sep_out = "-"), "a")
+  expect_equal(to_any_case(""), "")
+  expect_equal(to_any_case("a"), "a")
+  expect_equal(to_any_case(c("bla_bla_bla"), sep_out = c("-", "_")),  "bla-bla_bla")
+  
+  expect_equal(to_any_case(c("2018_01_01_bla_bla_bla"), sep_out = c("-", "_")), "2018-01_01_bla_bla_bla")
+  expect_equal(to_any_case(c("2018_01_01_bla_bla_bla"), sep_out = "-"), "2018-01-01-bla-bla-bla")
+  expect_equal(to_any_case(c("2018_01_01_bla_bla"), sep_out = c("-", "-", "_", "_", "_", "_", "_")),  "2018-01-01_bla_bla")
+  expect_equal(to_any_case(character(0), sep_out = c("_", "_")), character(0))
+})
+
+test_that("random case", {
+  expect_equal(
+    {suppressWarnings(RNGversion("3.1")); set.seed(123); to_any_case("almost RANDOM", case = "random")},
+    "AlMosT raNdOm"
+    )
+})
+
+test_that("title case", {
+  expect_equal(
+    to_any_case(c("on_andOn", "AndON", " and on", "and so on", "seems like it works", "also abbreviations ETC"), case = "title", abbreviations = "ETC"),
+    c("On and on", "And on", "And on", "And so on", "Seems Like it Works", "also Abbreviations ETC") 
+  )
+})
+
+test_that("case none", {
+  expect_equal(
+    to_any_case(c("blabla", "blablub", "blaBlub"),case = "none", transliterations = c(blab = "blub")),
+    c("blubla", "blublub", "blaBlub") 
+  )
+})
+
+test_that("special_input", {
+  expect_identical(to_any_case(NA_character_), NA_character_)
+  expect_equal(to_any_case(character(0)), character(0))
+})
